@@ -6,8 +6,10 @@ from pathlib import Path
 import os
 import time
 import socket
-
+from networktools.colorprint import bprint, gprint, rprint
 from datadbs.general import GeneralData
+from asyncio import shield, wait_for
+from rethinkdb.errors import RqlDriverError, RqlError
 
 
 class Rethink_DBS(GeneralData):
@@ -31,7 +33,7 @@ class Rethink_DBS(GeneralData):
         if self:
             self.save_log(msg, level)
 
-    async def async_connect(self, loop=None):
+    async def async_connect(self, loop=None, *args, **kwargs):
         self.session = None
         try:
             if self.default_db:
@@ -43,22 +45,32 @@ class Rethink_DBS(GeneralData):
                 dbname = self.dbname
                 dbhost = self.address[0]
                 dbport = self.address[1]
+                gprint("=" * 30)
+                bprint(
+                    f"Conectando {dbname}, {dbhost}, {dbport}, for station {self.station}"
+                )
+                gprint("=" * 30)
                 self.session = await self.r.connect(db=dbname,
                                                     host=dbhost,
                                                     port=dbport,
                                                     **kwargs)
-                self.logger.info("Connected to RethinkDB with database %s" %
-                                 self.default_db)
+                self.logger.info(
+                    f"Connected to RethinkDB with database {self.default_db}")
             else:
                 self.session = await self.r.connect(host=self.host,
                                                     port=self.port)
                 self.logger.info(
                     "Connected to RethinkDB without database selected")
+        except (RqlDriverError, RqlError) as r_error:
+            kwargs["origin_exception"] = RqlError
+            msg = f'''PD_CONN, ConnectionClosed {r_error} connection....'''
+            self.logger.exception(kwargs.get("origin_exception"))
+            raise r_error
         except (socket.error, ConnectionResetError,
                 ConnectionAbortedError) as conn_error:
+            kwargs["origin_exception"] = socket.error
             msg = f'''PD_TOA_-1 + {conn_error}, ConnectionClosed rc_enu connection....'''
-            self.logger.exception(kwargs["origin_exception"])
-            rc = None
+            self.logger.exception(kwargs.get("origin_exception"))
             raise conn_error
         except Exception as ex:
             self.logger.exception("Excepcion al conectar a Rethinkdb %s" % ex)
@@ -66,8 +78,8 @@ class Rethink_DBS(GeneralData):
         return self.session
 
     def close(self):
+        self.session.close(noreply_wait=True)
         super().close()
-        self.session.close(noreply_wait=False)
 
     async def reconnect(self, wait=False):
         await self.session.reconnect(noreply_wait=wait)
@@ -163,14 +175,13 @@ class Rethink_DBS(GeneralData):
                     return {'msg': msg}
         except Exception as e:
             self.logger.exception(
-                "Error crítico al crear tabla %s, exception" % (table_name, e))
+                "Error crítico al crear tabla %s, exception" % (e))
             raise e
 
     async def get_indexes(self, table_name: str, dbname: str = None):
         if not dbname:
             dbname = self.default_db
         await self.list_tables()
-        await self.list_tables()
         if table_name in self.tables.get(dbname, {}):
             indexes = await self.r.db(dbname).table(
                 table_name).index_list().run(self.session)
@@ -207,8 +218,8 @@ class Rethink_DBS(GeneralData):
                 return await self.create_table(table_name, dbname)
 
         except Exception as e:
-            self.logger.exception("Index %s can't be created %s, exception" %
-                                  (index, table_name, e))
+            self.logger.exception(
+                f"Index {index} can't be created {table_name}, exception {e}")
             raise e
 
     async def delete_table(self, table_name: str, dbname: str):
@@ -239,16 +250,26 @@ class Rethink_DBS(GeneralData):
         return changes
 
     async def save_data(self, table_name, data, options=None):
-        # print(self.tables)
+        task = asyncio.current_task()
+        task.set_name(f"save_data_{table_name}")
         try:
             if table_name in self.tables[self.defaultdb]:
-                # print("Sending data %s" %data)
-                if options is None:
-                    options = {'durability': 'soft', 'return_changes': False}
-                return await self.r.db(
-                    self.defaultdb).table(table_name).insert(
-                        data, **options).run(self.session, **options)
-        except Exception as ex:
+                task_coro = self.r.db(self.defaultdb).table(table_name).insert(
+                    data, durability='soft',
+                    return_changes=True).run(self.session)
+                return await task_coro
+        except asyncio.CancelledError as ce:
+            task = asyncio.current_task()
+            if task:
+                result = task.result()
+            self.logger.exception(
+                "Error de cancelación de tarea {result},{table_name}")
+            bprint(
+                f"Se cancela tarea para {table_name}, {filter_opt}, {result}")
+        except Exception as ec:
+            task = asyncio.current_task()
+            if task:
+                result = task.result()
             self.logger.critical(
                 "Error in saving data on %s with data %s, error %s" %
                 (table_name, data, ex))
@@ -290,8 +311,10 @@ class Rethink_DBS(GeneralData):
 
     async def get_data_between(self, table_name, lower, upper):
         dbname = self.dbname
-        self.logger.info("Extracting data between %s and %s from %s" %
-                         (lower, upper, table_name))
+        msg = "Extracting data between %s and %s from %s" % (lower, upper,
+                                                             table_name)
+        self.logger.info(msg)
+        bprint(msg)
         if table_name in self.tables[self.default_db]:
             return await self.r.db(dbname).table(table_name).between(
                 lower, upper).run(self.session)
@@ -305,7 +328,7 @@ class Rethink_DBS(GeneralData):
     async def get_data_filter(self,
                               table_name,
                               filter_exp,
-                              filter_opt,
+                              filter_opt: dict,
                               order_by: str,
                               options={}):
         """
@@ -314,19 +337,17 @@ class Rethink_DBS(GeneralData):
         r.row['age']=30 or a lambda function:
         lambda user: user['age]==30
         """
+        task = asyncio.current_task()
+        if task:
+            task.set_name(f"get_data_filter_{table_name}")
         dbname = self.dbname
-        index = None
-        if "index" in filter_opt.keys():
-            index = filter_opt["index"]
-            if not index:
-                index = None
+        index = filter_opt.get('index')
         if table_name in self.tables.get(self.default_db, {}):
             options = {'read_mode': 'single'}
 
         try:
             if table_name in self.tables.get(self.default_db, {}):
                 if index:
-                    # print("Between")
                     lower = None
                     upper = None
                     if not type(filter_exp) == list:
@@ -335,25 +356,54 @@ class Rethink_DBS(GeneralData):
                     else:
                         lower = filter_exp[0]
                         upper = filter_exp[1]
-                    self.logger.info(
-                        "Obtaining data from %s to %s on table %s, with filters %s using index %s"
-                        % (lower, upper, table_name, filter_opt, index))
-                    return await self.r.db(dbname).table(table_name).between(
-                        lower, upper, **filter_opt).order_by(order_by).run(
-                            self.session, **options)
+                    query = rdb.db(dbname).table(table_name).between(
+                        lower,
+                        upper,
+                        index=index,
+                        left_bound="open",
+                        right_bound="open").coerce_to("array").order_by(
+                            index).run(self.session)
+                    result = await query
+                    return result
                 else:
                     self.logger.info(
                         "Obtaining data with filters %s using table %s" %
                         (filter_opt, table_name))
 
                     return await self.r.db(dbname).table(table_name).filter(
-                        filter_exp).order_by(order_by).run(
+                        filter_exp).order_by(order_by).coerce_to("array").run(
                             self.session, **options)
             else:
                 return []
+        except asyncio.InvalidStateError as ie:
+            rprint(f"Invalida state task {task}")
+            task = asyncio.current_task()
+            if task:
+                task.cancel()
+                result = task.result()
+            self.logger.exception("Error de cancelación de tarea {result}")
+            bprint(
+                f"Se cancela tarea para {table_name}, {filter_opt}, {result}")
+            raise ie
+        except asyncio.CancelledError as ce:
+            task = asyncio.current_task()
+            if task:
+                task.cancel()
+                result = task.result()
+            self.logger.exception("Error de cancelación de tarea {result}")
+            bprint(
+                f"Se cancela tarea para {table_name}, {filter_opt}, {result}")
+            raise ce
         except Exception as ec:
-            self.logger.exception("Error on the data extraction, error %s" %
-                                  ec)
+            task = asyncio.current_task()
+            if task:
+                task.cancel()
+                result = task.result()
+            bprint(
+                f"Se cancela tarea para {table_name}, {filter_opt}, result {result}"
+            )
+            self.logger.exception(
+                f"Error on the data extraction, error {ec}, result {result}")
             await asyncio.sleep(5)
             await self.reconnect(wait=True)
             raise ec
